; ---------------------------------------------------------------------------
; Wetflame screen. (Main Code)
; ---------------------------------------------------------------------------

Wetflame_Main:
		move.b	#$E4, d0
		jsr	PlaySound	; Stop the music.
		
		; Clear everything. (Code provided by Sonic65.)
WTF_ClearEverything:
		jsr	ClearPLC
		jsr	Pal_FadeFrom
		clr.b	($FFFFF64E).w
		move.w	#$2700, sr		; Disable interrupts.
		move.w	($FFFFF60C).w, d0
		move.w	d0, d1
		andi.b	#$BF, d0
		move.w	d0, ($C00004).l
		jsr	ClearScreen
		
		move.b	#0,($FFFFFE10).w
		move.b	#0,($FFFFFE11).w
		
; --------------------------------------------------------------
		
		; Load the PCM driver.
		move.w	#$100,($A11100).l	; Stop the Z80.
WTF_PlayMusic:
		btst	#0,($A11100).l
		bne.s	WTF_PlayMusic		; Wait for the Z80 to halt.
		jsr	PCM_DriverLoad
		
		; Initialize the PCM driver.
		move.b	#1,($a00039).l					; Turn PCM Playing Flag ON.
		move.b	#(((MusicWTF+$34)&0xFF0000)>>16),($a0003c).l		; addr - $__xxxx
		move.b	#(((MusicWTF+$34)&0xFF00)>>8),($a0003b).l		; addr - $xx__xx
		move.b	#((MusicWTF+$34)&0xFF),($a0003a).l			; addr - $xxxx__
		
		move.b	#(((MusicWTFEnd-MusicWTF)&0xFF0000)>>16),($a0003f).l	; addr - $__xxxx
		move.b	#(((MusicWTFEnd-MusicWTF)&0xFF00)>>8),($a0003e).l	; addr - $xx__xx
		move.b	#((MusicWTFEnd-MusicWTF)&0xFF),($a0003d).l		; addr - $xxxx__
		
		move.b	#(((MusicWTF+$34)&0xFF0000)>>16),($A0004E).l
		move.b	#(((MusicWTF+$34)&0xFF00)>>8),($A0004D).l
		move.b	#((MusicWTF+$34)&0xFF),($A0004C).l
		
		move.b	#(((MusicWTFEnd-MusicWTF)&0xFF0000)>>16),($a00051).l	; addr - $__xxxx
		move.b	#(((MusicWTFEnd-MusicWTF)&0xFF00)>>8),($a00050).l	; addr - $xx__xx
		move.b	#((MusicWTFEnd-MusicWTF)&0xFF),($a0004F).l		
		
		move.b	#$00,($a00046).l	; sample rate (00=16000Hz)
		move.b	#$01,($a0004A).l	; looping flag ($00=no,$01=yes)
		
		move.w	#$0,($A11100).l		; Start the Z80.
		
; --------------------------------------------------------------
		
		; More clearing.
		moveq	#0, d0
		lea	($C00004).l, a5		; VRAM control port.
		lea	($C00000).l, a6		; VRAM data port.
		move.l	#$40000000, (a5)	; Clear VRAM.
		move.w	#$3FFF, d1		; $4000 DWORDs to clear.
		
WTF_ClrVRAM:
		move.l	d0, (a6)
		dbf	d1, WTF_ClrVRAM
		
		lea	($FFFFCC00).w, a1
		move.w	#$DF, d1
		
WTF_ClrScroll:
		move.l	d0, (a1)+
		dbf	d1, WTF_ClrScroll	; Clear the scroll data.
		
		lea	($FFFFD000).w, a1
		move.w	#$800, d1
		
WTF_ClrObjects:
		move.l	d0, (a1)+
		dbf	d1, WTF_ClrObjects	; Clear the object data.
		
; --------------------------------------------------------------
		
		; Load the C64 character set.
		moveq	#$2, d0			; Color: $2
		jsr	C64_charset_load
		
; --------------------------------------------------------------
		
		; Set up the scroll plane.
		move.w	#$9001, (a5)		; Scroll size: 64x32
		move.w	#-225, d7		; Start at -225 so the first line can be loaded.
		move.l	#$40000010, (a5)
		move.w	d7, (a6)
		
; --------------------------------------------------------------
		
		; Reserved registers:
		; - d4: Frame counter.
		; - d5: Current scroll line.
		; - d6: Wetflame TL;DR line counter.
		; - d7: Current VScroll value.
		; - a1: Wetflame_TLDR address.
		
		; Load the palette.
		move.l	#$C0040000, (a5)	; Palette 0, Index 2.
		move.w	#$0EEE, (a6)
		
		; Pick a random Wetflame TL;DR.
		move.w	($FFFFFE0E).w,d0
		andi.w	#$1F,d0
		
		; Load the TL;DR's address.
		lea	(TLDR_Table), a1
		lsl.w	#2, d0
		movea.l	(a1,d0.w),a1
		
		; Initialize the TL;DR variables.
		move.w	(a1)+, d6	; Number of lines for the TL;DR.
		moveq	#1, d5		; Current scroll line.
		moveq	#0, d4		; Frame counter.
		
		; Enable the display.
		move.w	#$8164, ($C00004).l
		
; --------------------------------------------------------------
		
		; Start of Wetflame!
		; TODO: Replace with Sonic65's PCM driver.
		
		; Note: Interrupts are disabled due to a conflict
		; with the text scrolling if Labyrinth Zone was the
		; active zone when a Wetflame monitor was broken.
		;move.w	#$2000, sr
		
WTF_scroll_loop:
		
		; VBlank routine by Sik.
VBlank_Wait_1:
		; Wait for current VBlank to exit.
		move.w	($C00004).l, d0
		btst.l	#3, d0
		bne.s	VBlank_Wait_1

VBlank_Wait_2:
		; Wait for next VBlank to start.
		move.w	($C00004).l, d0
		btst.l	#3, d0
		beq.s	VBlank_Wait_2
		
		; Next frame.
		addq	#1, d4		; Frame counter.
		btst	#0, d4		; Update on even frames only.
		bne.s	WTF_scroll_loop
		
		; Next frame. Update the VScroll.
		addq	#1, d7
		move.l	#$40000010, (a5)
		move.w	d7, (a6)
		
		; Check if a scroll line needs to be written.
		move.w	d7, d0
		andi.b	#7, d0
		bne.s	WTF_scroll_loop
		
		; VScroll is a multiple of 8.
		move.l	d5, d0		; Set up the Scroll A write address.
		lsl.w	#7, d0		; Shift left by 7 bits. (64 cells horizontally, times two.)
		ori.w	#$4000, d0	; Part of Scroll A's VRAM address.
		swap	d0		; Swap the words in d0.
		ori.b	#$3, d0		; Part of Scroll A's VRAM address.
		move.l	d0, (a5)	; Set the VRAM address.
		
		; Increment the scroll line for next time.
		addq	#1, d5		; Next scroll line.
		andi	#$1F, d5	; 32 scroll lines.
		
		; Check if the TL;DR is completely offscreen.
		cmpi.w	#-35, d6
		beq.s	WTF_finished
		
		; Check if there are any more lines to write.
		cmpi.w	#0, d6
		ble.s	WTF_write_blank_line
		
		; Write a line of text.
		; d0 == column; d1 == data.
		moveq	#39, d0		; 40 columns.
		moveq	#0, d1		; Clear the data register.
		
TLDR_Load:
		; Load a character from the TL;DR.
		move.b	(a1)+, d1
		move.w	d1, (a6)	; Write the character to VRAM.
		dbf	d0, TLDR_Load	; Next column.
		subq	#1, d6		; Decrement the line counter.
		bra.s	WTF_scroll_loop
		
WTF_write_blank_line:
		; Write a blank line to VRAM.
		; d0 == column; d1 == data.
		moveq	#$F, d0		; 64 columns, four at a time.
		moveq	#0, d1		; Clear the data register.
		subq	#1, d6		; Decrement the line counter.
		
WTF_write_blank_line_loop:
		move.l	d1, (a6)	; Write the blank characters to VRAM.
		move.l	d1, (a6)	; Write the blank characters to VRAM.
		dbf	d0, WTF_write_blank_line_loop
		bra.s	WTF_scroll_loop
		
; --------------------------------------------------------------
		
WTF_finished:
		; Finished scrolling. Lose the game.
		jsr	Pal_FadeTo
		jmp	GameScreen
		
; ===========================================================================
		
		; Data for the TL;DR screen.
		include C64_charset.asm
		include	tldr.asm
		
; ===========================================================================

MusicWTF:
		; "Benny Hill" theme.
		even
		incbin	wtflame/sound/bennyHill.bin
		even
MusicWTFEnd:

; ===========================================================================

; PCM driver.
PCM_DriverLoad:					; XREF: Wetflame_Main
		move.w	#0,($A11200).l
		move.w	#$100,($A11100).l
		move.w	#$100,($A11200).l

WaitForZ80_2:
		btst	#0,($A11100).l
		bne.s	WaitForZ80_2

		lea	(Z802),a0
		lea	($A00000).l,a1
		move.w	#Z802End-Z802,d1

LoadDriver:
		move.b	(a0)+,(a1)+
		dbf	d1,LoadDriver
		move.b	#$01,($FFFFFFFC).w
		rts

; ===========================================================================

Z802:
		incbin wtflame/sound/PCMD.bin
		even
Z802End:

; ===========================================================================

; ---------------------------------------------------------------------------
; YOU JUST LOST THE GAME
; ---------------------------------------------------------------------------

GameScreen:				; XREF: Wetflame_Main
		jsr		Pal_FadeFrom
		
		move.l	#$C0040000, ($C00004).l	; Palette 0, Index 2.
		move.w	#$0000, ($C00000).l
		move.w	#$2700,sr
		clr.b	($FFFFF64E).w
		move.w	($FFFFF60C).w,d0
		andi.b	#$BF,d0
		move.w	d0,($C00004).l
		jsr	ClearScreen
		
		move.l	#$40000000,($C00004).l
		lea	(Nem_GameScreen).l,a0	; Load "The Game" screen patterns.
		jsr	NemDec
		lea	($FF0000).l,a1
		lea	(Eni_GameScreen).l,a0	; Load "The Game" screen mappings.
		move.w	#0,d0
		jsr	EniDec
		lea	($FF0000).l,a1
		move.l	#$40000003,d0
		moveq	#$27,d1
		moveq	#$1B,d2
		jsr	ShowVDPGraphics
		
		move.w	#-$A,($FFFFF632).w
		move.w	#0,($FFFFF634).w
		move.w	#0,($FFFFF662).w
		move.w	#0,($FFFFF660).w
		move.w	($FFFFF60C).w,d0
		ori.b	#$40,d0
		move.w	d0,($C00004).l
		
		move.w	#$100,($A11100).l	; Stop the Z80.
GameScreen_StopZ80:
		btst	#0,($A11100).l
		bne.s	GameScreen_StopZ80	; Wait for the Z80 to halt.
		
		; Reload the Sonic 1 sound driver and play the "Game Over" theme.
		jsr SoundDriverLoad
		move.b	#$8F,d0
		jsr	PlaySound
		
		moveq	#21,d0
		jsr	PalLoad1	; load Sega logo pallet
		jsr	Pal_FadeTo
		
		moveq	#0,d0

GameScreen_Wait:
		; Wait a while.
		move.b	#8,($FFFFF62A).w
		jsr		DelayProgram
		
		; Check if we've waited long enough.
		add.w	#1,d0
		cmpi.w	#$300,d0
		bne.s	GameScreen_Wait	; if not, branch
		
GameScreen_HardReset:	
		jsr	Pal_FadeFrom
		movea.l	($00000000).w, a7	; Load initial SP.
		movea.l	($00000004).w, a0	; Load initial PC.
		jmp	(a0)			; Jump to initial PC.
		
Nem_GameScreen:
		incbin	wtflame/gameTiles.bin
		even

Eni_GameScreen:
		incbin	wtflame/gameMap.bin
		even
; ===========================================================================

; ---------------------------------------------------------------------------
; FIRST @ TSSZ
; ---------------------------------------------------------------------------

TSSZScreen:				; XREF: GameModeArray
		; Stop the music.
		move.b	#$E4,d0
		jsr	PlaySound_Special
		
		jsr	Pal_FadeFrom
		jsr	ClearPLC
		
		move.l	#$C0040000, ($C00004).l	; Palette 0, Index 2.
		move.w	#$0000, ($C00000).l
		
		clr.b	($FFFFF64E).w
		move	#$2700,sr
		move.w	($FFFFF60C).w,d0
		andi.b	#$BF,d0
		move.w	d0,($C00004).l
		jsr	ClearScreen
		
		move.l	#$40000000,($C00004).l
		lea	(Nem_TSSZScreen).l,a0 ; load Sega	logo patterns
		jsr	NemDec
		lea	($FF0000).l,a1
		lea	(Eni_TSSZScreen).l,a0 ; load Sega	logo mappings
		move.w	#0,d0
		jsr	EniDec
		lea	($FF0000).l,a1
		move.l	#$40000003,d0
		moveq	#$27,d1
		moveq	#$1B,d2
		jsr	ShowVDPGraphics
		
		move.w	#-$A,($FFFFF632).w
		move.w	#0,($FFFFF634).w
		move.w	#0,($FFFFF662).w
		move.w	#0,($FFFFF660).w
		move.w	($FFFFF60C).w,d0
		ori.b	#$40,d0
		move.w	d0,($C00004).l
		
		; PCM stuff
		;move.w #$100,($A11100).l ;Stop the Z80
		
;TSSZScreen_StopZ80:
;		btst #0,($A11100).l
;		bne TSSZScreen_StopZ80 ;Wait for z80 to halt		
		
		; Play the "End of Level" theme.
		jsr SoundDriverLoad
		move.b	#$8E,d0
		jsr	PlaySound
				
		moveq	#22,d0
		jsr	PalLoad1	; load Sega logo pallet
		jsr	Pal_FadeTo
		
		moveq	#0,d0

TSSZScreen_Wait:
		; Wait a while.
		move.b	#8,($FFFFF62A).w
		jsr	DelayProgram
		
		andi.b	#$F0,($FFFFF605).w	; Are A, B, C, or Start pressed?
		bne.s	TSSZScreen_WaitEnd	; if yes, branch
		
		; Check if we've waited long enough.
		add.w	#1,d0
		cmpi.w	#$180,d0
		bne.s	TSSZScreen_Wait	; if not, branch
		
TSSZScreen_WaitEnd:
		move.b	#4,($FFFFF600).w ; go to title screen
		jmp	TitleScreen
		rts
		
Nem_TSSZScreen:
		incbin	wtflame/tsszTiles.bin
		even

Eni_TSSZScreen:
		incbin	wtflame/tsszMap.bin
		even

; ===========================================================================

; ---------------------------------------------------------------------------
; SONIC 65 AND GERBILSOFT PRESENT
; ---------------------------------------------------------------------------

PresentScreen:				; XREF: GameModeArray
		; Stop the music.
		move.b	#$E4,d0
		jsr	PlaySound_Special
		
		jsr	Pal_FadeFrom
		jsr	ClearPLC
		
		move.l	#$C0040000, ($C00004).l	; Palette 0, Index 2.
		move.w	#$0000, ($C00000).l

		clr.b	($FFFFF64E).w
		move	#$2700,sr
		move.w	($FFFFF60C).w,d0
		andi.b	#$BF,d0
		move.w	d0,($C00004).l
		jsr	ClearScreen
		
		move.l	#$40000000,($C00004).l
		lea	(Nem_PresentScreen).l,a0	; Load the "Presents" screen patterns.
		jsr	NemDec
		lea	($FF0000).l,a1
		lea	(Eni_PresentScreen).l,a0	; Load the "Presents" screen mappings.
		move.w	#0,d0
		jsr	EniDec
		lea	($FF0000).l,a1
		move.l	#$40000003,d0
		moveq	#$27,d1
		moveq	#$1B,d2
		jsr	ShowVDPGraphics
		
		move.w	#-$A,($FFFFF632).w
		move.w	#0,($FFFFF634).w
		move.w	#0,($FFFFF662).w
		move.w	#0,($FFFFF660).w
		move.w	($FFFFF60C).w,d0
		ori.b	#$40,d0
		move.w	d0,($C00004).l
		
		lea	($FFFFFB80).w,a1
		moveq	#0,d0
		move.w	#$1F,d1
		
PresentScreen_ClrPallet:
		move.l	d0,(a1)+
		dbf	d1,PresentScreen_ClrPallet ; fill pallet with 0	(black)
		
		; TODO: PCM driver.
		;move.w	#$100,($A11100).l ;Stop the Z80
		
;PresentScreen_StopZ80:
;		btst #0,($A11100).l
;		bne PresentScreen_StopZ80 ;Wait for z80 to halt		

		moveq	#23,d0
		jsr	PalLoad1	; load Sega logo pallet
		jsr	Pal_FadeTo
		
		moveq	#0,d0
		
PresentScreen_Wait:
		; Wait a while.
		move.b	#8,($FFFFF62A).w
		jsr	DelayProgram
		
		andi.b	#$F0,($FFFFF605).w	; Are A, B, C, or Start pressed?
		bne.s	PresentScreen_WaitEnd	; if yes, branch
		
		; Check if we've waited long enough.
		add.w	#1,d0
		cmpi.w	#$20,d0
		bne.s	PresentScreen_Wait	; if not, branch
		
PresentScreen_WaitEnd:
		move.b	#4,($FFFFF600).w ; go to title screen
		rts
		
Nem_PresentScreen:
		incbin	wtflame/sgpTiles.bin
		even

Eni_PresentScreen:
		incbin	wtflame/sgpMap.bin
		even
; ===========================================================================
